@@ -4,10 +4,11 @@ import logging
 import os
 import os.path
 import configparser
+from dataclasses import dataclass
 
 import pluggy
 from itertools import chain
-from typing import Dict, Iterator, List, Tuple, Any, Optional, Union, Iterable
+from typing import Dict, Iterator, List, Tuple, Any, Optional, Union, Iterable, Callable
 from pathlib import Path
 from sqlfluff.core.plugin.host import get_plugin_manager
 from sqlfluff.core.errors import SQLFluffUserError
@@ -16,7 +17,7 @@ import appdirs
 
 import toml
 
-# Instantiate the templater logger
+# Instantiate the config logger
 config_logger = logging.getLogger("sqlfluff.config")
 
 global_loader = None
@@ -28,9 +29,52 @@ can still cache appropriately
 
 ConfigElemType = Tuple[Tuple[str, ...], Any]
 
-REMOVED_CONFIGS: Dict[Tuple[str, ...], Any] = {
-    ("rules", "L003", "lint_templated_tokens"): ("No longer used."),
-}
+
+@dataclass
+class _RemovedConfig:
+    old_path: Tuple[str, ...]
+    warning: str
+    new_path: Optional[Tuple[str, ...]] = None
+    translation_func: Optional[Callable[[str], str]] = None
+
+
+REMOVED_CONFIGS = [
+    _RemovedConfig(
+        ("rules", "L007", "operator_new_lines"),
+        (
+            "Use the line_position config in the appropriate "
+            "sqlfluff:layout section (e.g. sqlfluff:layout:type"
+            ":binary_operator)."
+        ),
+        ("layout", "type", "binary_operator", "line_position"),
+        (lambda x: "trailing" if x == "before" else "leading"),
+    ),
+    _RemovedConfig(
+        ("rules", "comma_style"),
+        (
+            "Use the line_position config in the appropriate "
+            "sqlfluff:layout section (e.g. sqlfluff:layout:type"
+            ":comma)."
+        ),
+        ("layout", "type", "comma", "line_position"),
+        (lambda x: x),
+    ),
+    # L019 used to have a more specific version of the same /config itself.
+    _RemovedConfig(
+        ("rules", "L019", "comma_style"),
+        (
+            "Use the line_position config in the appropriate "
+            "sqlfluff:layout section (e.g. sqlfluff:layout:type"
+            ":comma)."
+        ),
+        ("layout", "type", "comma", "line_position"),
+        (lambda x: x),
+    ),
+    _RemovedConfig(
+        ("rules", "L003", "lint_templated_tokens"),
+        "No longer used.",
+    ),
+]
 
 
 def coerce_value(val: str) -> Any:
@@ -71,6 +115,13 @@ def nested_combine(*dicts: dict) -> dict:
     Returns:
         `dict`: A combined dictionary from the input dictionaries.
 
+    A simple example:
+    >>> nested_combine({"a": {"b": "c"}}, {"a": {"d": "e"}})
+    {'a': {'b': 'c', 'd': 'e'}}
+
+    Keys overwrite left to right:
+    >>> nested_combine({"a": {"b": "c"}}, {"a": {"b": "e"}})
+    {'a': {'b': 'e'}}
     """
     r: dict = {}
     for d in dicts:
@@ -105,10 +156,20 @@ def dict_diff(left: dict, right: dict, ignore: Optional[List[str]] = None) -> di
         left (:obj:`dict`): The object containing the *new* elements
             which will be compared against the other.
         right (:obj:`dict`): The object to compare against.
+        ignore (:obj:`list` of `str`, optional): Keys to ignore.
 
     Returns:
         `dict`: A dictionary representing the difference.
 
+    Basic functionality shown, especially returning the left as:
+    >>> dict_diff({"a": "b", "c": "d"}, {"a": "b", "c": "e"})
+    {'c': 'd'}
+
+    Ignoring works on a key basis:
+    >>> dict_diff({"a": "b"}, {"a": "c"})
+    {'a': 'b'}
+    >>> dict_diff({"a": "b"}, {"a": "c"}, ["a"])
+    {}
     """
     buff: dict = {}
     for k in left:
@@ -146,7 +207,7 @@ class ConfigLoader:
 
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         # TODO: check that this cache implementation is actually useful
         self._config_cache: dict = {}
 
@@ -173,7 +234,15 @@ class ConfigLoader:
 
     @classmethod
     def _iter_config_elems_from_dict(cls, configs: dict) -> Iterator[ConfigElemType]:
-        """Walk a config dict and get config elements."""
+        """Walk a config dict and get config elements.
+
+        >>> list(
+        ...    ConfigLoader._iter_config_elems_from_dict(
+        ...        {"foo":{"bar":{"baz": "a", "biz": "b"}}}
+        ...    )
+        ... )
+        [(('foo', 'bar', 'baz'), 'a'), (('foo', 'bar', 'biz'), 'b')]
+        """
         for key, val in configs.items():
             if isinstance(val, dict):
                 for partial_key, sub_val in cls._iter_config_elems_from_dict(val):
@@ -181,6 +250,25 @@ class ConfigLoader:
             else:
                 yield (key,), val
 
+    @classmethod
+    def _config_elems_to_dict(cls, configs: Iterable[ConfigElemType]) -> dict:
+        """Reconstruct config elements into a dict.
+
+        >>> ConfigLoader._config_elems_to_dict(
+        ...     [(("foo", "bar", "baz"), "a"), (("foo", "bar", "biz"), "b")]
+        ... )
+        {'foo': {'bar': {'baz': 'a', 'biz': 'b'}}}
+        """
+        result: Dict[str, Union[dict, str]] = {}
+        for key, val in configs:
+            ref = result
+            for step in key[:-1]:
+                if step not in ref:
+                    ref[step] = {}
+                ref = ref[step]  # type: ignore
+            ref[key[-1]] = val
+        return result
+
     @classmethod
     def _get_config_elems_from_toml(cls, fpath: str) -> List[ConfigElemType]:
         """Load a config from a TOML file and return a list of tuples.
@@ -261,7 +349,13 @@ class ConfigLoader:
 
     @staticmethod
     def _incorporate_vals(ctx: dict, vals: List[ConfigElemType]) -> dict:
-        """Take a list of tuples and incorporate it into a dictionary."""
+        """Take a list of tuples and incorporate it into a dictionary.
+
+        >>> ConfigLoader._incorporate_vals({}, [(("a", "b"), "c")])
+        {'a': {'b': 'c'}}
+        >>> ConfigLoader._incorporate_vals({"a": {"b": "c"}}, [(("a", "d"), "e")])
+        {'a': {'b': 'c', 'd': 'e'}}
+        """
         for k, v in vals:
             # Keep a ref we can use for recursion
             r = ctx
@@ -284,17 +378,65 @@ class ConfigLoader:
         return ctx
 
     @staticmethod
-    def _validate_configs(configs: Iterable[ConfigElemType], file_path):
+    def _validate_configs(
+        configs: Iterable[ConfigElemType], file_path
+    ) -> List[ConfigElemType]:
         """Validate config elements against removed list."""
-        for k, _ in configs:
-            if k in REMOVED_CONFIGS:
+        config_map = {cfg.old_path: cfg for cfg in REMOVED_CONFIGS}
+        # Materialise the configs into a list to we can iterate twice.
+        new_configs = list(configs)
+        defined_keys = {k for k, _ in new_configs}
+        validated_configs = []
+        for k, v in new_configs:
+            if k in config_map.keys():
                 formatted_key = ":".join(k)
-                raise SQLFluffUserError(
-                    f"Config file {file_path} set an outdated config "
-                    f"value {formatted_key}.\n\n{REMOVED_CONFIGS[k]}\n\n"
-                    "See https://docs.sqlfluff.com/en/stable/configuration.html"
-                    " for more details."
-                )
+                removed_option = config_map[k]
+                # Is there a mapping option?
+                if removed_option.translation_func and removed_option.new_path:
+                    formatted_new_key = ":".join(removed_option.new_path)
+                    # Before mutating, check we haven't _also_ set the new value.
+                    if removed_option.new_path in defined_keys:
+                        # Raise an warning.
+                        config_logger.warning(
+                            f"\nWARNING: Config file {file_path} set a deprecated "
+                            f"config value `{formatted_key}` (which can be migrated) "
+                            f"but ALSO set the value it would be migrated to. The new "
+                            f"value (`{removed_option.new_path}`) takes precedence. "
+                            "Please update your configuration to remove this warning. "
+                            f"\n\n{removed_option.warning}\n\n"
+                            "See https://docs.sqlfluff.com/en/stable/configuration.html"
+                            " for more details.\n"
+                        )
+                        # continue to NOT add this value in the set
+                        continue
+
+                    # Mutate and warn.
+                    v = removed_option.translation_func(v)
+                    k = removed_option.new_path
+                    # NOTE: At the stage of emitting this warning, we may not yet
+                    # have set up red logging because we haven't yet loaded the config
+                    # file. For that reason, this error message has a bit more padding.
+                    config_logger.warning(
+                        f"\nWARNING: Config file {file_path} set a deprecated config "
+                        f"value `{formatted_key}`. This will be removed in a later "
+                        "release. This has been mapped to "
+                        f"`{formatted_new_key}` set to a value of `{v}` for this run. "
+                        "Please update your configuration to remove this warning. "
+                        f"\n\n{removed_option.warning}\n\n"
+                        "See https://docs.sqlfluff.com/en/stable/configuration.html"
+                        " for more details.\n"
+                    )
+                else:
+                    # Raise an error.
+                    raise SQLFluffUserError(
+                        f"Config file {file_path} set an outdated config "
+                        f"value {formatted_key}.\n\n{removed_option.warning}\n\n"
+                        "See https://docs.sqlfluff.com/en/stable/configuration.html"
+                        " for more details."
+                    )
+
+            validated_configs.append((k, v))
+        return validated_configs
 
     def load_config_file(
         self, file_dir: str, file_name: str, configs: Optional[dict] = None
@@ -305,7 +447,7 @@ class ConfigLoader:
             elems = self._get_config_elems_from_toml(file_path)
         else:
             elems = self._get_config_elems_from_file(file_path)
-        self._validate_configs(elems, file_path)
+        elems = self._validate_configs(elems, file_path)
         return self._incorporate_vals(configs or {}, elems)
 
     def load_config_at_path(self, path: str) -> dict:
@@ -497,13 +639,15 @@ class FluffConfig:
         )
         # If overrides are provided, validate them early.
         if overrides:
-            ConfigLoader._validate_configs(
-                [
-                    (("core",) + k, v)
-                    for k, v in ConfigLoader._iter_config_elems_from_dict(overrides)
-                ],
-                "<provided overrides>",
-            )
+            overrides = ConfigLoader._config_elems_to_dict(
+                ConfigLoader._validate_configs(
+                    [
+                        (("core",) + k, v)
+                        for k, v in ConfigLoader._iter_config_elems_from_dict(overrides)
+                    ],
+                    "<provided overrides>",
+                )
+            )["core"]
         self._overrides = overrides  # We only store this for child configs
 
         # Fetch a fresh plugin manager if we weren't provided with one
@@ -512,8 +656,11 @@ class FluffConfig:
         defaults = nested_combine(*self._plugin_manager.hook.load_default_config())
         # If any existing configs are provided. Validate them:
         if configs:
-            ConfigLoader._validate_configs(
-                ConfigLoader._iter_config_elems_from_dict(configs), "<provided configs>"
+            configs = ConfigLoader._config_elems_to_dict(
+                ConfigLoader._validate_configs(
+                    ConfigLoader._iter_config_elems_from_dict(configs),
+                    "<provided configs>",
+                )
             )
         self._configs = nested_combine(
             defaults, configs or {"core": {}}, {"core": overrides or {}}
@@ -522,26 +669,21 @@ class FluffConfig:
         self._configs["core"]["color"] = (
             False if self._configs["core"].get("nocolor", False) else None
         )
-        # Deal with potential ignore parameters
-        if self._configs["core"].get("ignore", None):
-            self._configs["core"]["ignore"] = _split_comma_separated_string(
-                self._configs["core"]["ignore"]
-            )
-        else:
-            self._configs["core"]["ignore"] = []
-        # Allowlists and denylists
-        if self._configs["core"].get("rules", None):
-            self._configs["core"]["rule_allowlist"] = _split_comma_separated_string(
-                self._configs["core"]["rules"]
-            )
-        else:
-            self._configs["core"]["rule_allowlist"] = None
-        if self._configs["core"].get("exclude_rules", None):
-            self._configs["core"]["rule_denylist"] = _split_comma_separated_string(
-                self._configs["core"]["exclude_rules"]
-            )
-        else:
-            self._configs["core"]["rule_denylist"] = None
+        # Handle inputs which are potentially comma separated strings
+        for in_key, out_key in [
+            # Deal with potential ignore & warning parameters
+            ("ignore", "ignore"),
+            ("warnings", "warnings"),
+            ("rules", "rule_allowlist"),
+            # Allowlists and denylists
+            ("exclude_rules", "rule_denylist"),
+        ]:
+            if self._configs["core"].get(in_key, None):
+                self._configs["core"][out_key] = _split_comma_separated_string(
+                    self._configs["core"][in_key]
+                )
+            else:
+                self._configs["core"][out_key] = []
         # Configure Recursion
         if self._configs["core"].get("recurse", 0) == 0:
             self._configs["core"]["recurse"] = True
