@@ -4,7 +4,18 @@ import fnmatch
 import os
 import time
 import logging
-from typing import Any, List, Sequence, Optional, Tuple, cast, Iterable, Iterator, Set
+from typing import (
+    Any,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Type,
+    cast,
+)
 
 import pathspec
 import regex
@@ -16,6 +27,7 @@ from sqlfluff.core.errors import (
     SQLLintError,
     SQLParseError,
     SQLFluffSkipFile,
+    SQLFluffUserError,
 )
 from sqlfluff.core.parser import Lexer, Parser, RegexLexer
 from sqlfluff.core.file_helpers import get_encoding
@@ -59,7 +71,7 @@ class Linter:
         formatter: Any = None,
         dialect: Optional[str] = None,
         rules: Optional[List[str]] = None,
-        user_rules: Optional[List[BaseRule]] = None,
+        user_rules: Optional[List[Type[BaseRule]]] = None,
         exclude_rules: Optional[List[str]] = None,
     ) -> None:
         # Store the config object
@@ -99,7 +111,7 @@ class Linter:
     # These are the building blocks of the linting process.
 
     @staticmethod
-    def _load_raw_file_and_config(
+    def load_raw_file_and_config(
         fname: str, root_config: FluffConfig
     ) -> Tuple[str, FluffConfig, str]:
         """Load a raw file and the associated config."""
@@ -570,6 +582,7 @@ class Linter:
                         templated_file=templated_file,
                         ignore_mask=ignore_buff,
                         fname=fname,
+                        config=config,
                     )
                     if is_first_linter_pass():
                         initial_linting_errors += linting_errors
@@ -585,7 +598,7 @@ class Linter:
                                 f"Rule {crawler.code} returned conflicting "
                                 "fixes with the same anchor. This is only "
                                 "supported for create_before+create_after, so "
-                                "the fixes will not be applied. {fixes!r}"
+                                f"the fixes will not be applied. {fixes!r}"
                             )
                             cls._report_conflicting_fixes_same_anchor(message)
                             for lint_result in linting_errors:
@@ -719,10 +732,12 @@ class Linter:
         # We process the ignore config here if appropriate
         for violation in violations:
             violation.ignore_if_in(parsed.config.get("ignore"))
+            violation.warning_if_in(parsed.config.get("warnings"))
 
         linted_file = LintedFile(
             parsed.fname,
-            violations,
+            # Deduplicate violations
+            LintedFile.deduplicate_in_source_space(violations),
             time_dict,
             tree,
             ignore_mask=ignore_buff,
@@ -825,7 +840,7 @@ class Linter:
     def render_file(self, fname: str, root_config: FluffConfig) -> RenderedFile:
         """Load and render a file with relevant config."""
         # Load the raw file.
-        raw_file, config, encoding = self._load_raw_file_and_config(fname, root_config)
+        raw_file, config, encoding = self.load_raw_file_and_config(fname, root_config)
         # Render the file
         return self.render_string(raw_file, fname, config, encoding)
 
@@ -957,7 +972,9 @@ class Linter:
             if ignore_non_existent_files:
                 return []
             else:
-                raise OSError("Specified path does not exist")
+                raise SQLFluffUserError(
+                    f"Specified path does not exist. Check it/they exist(s): {path}."
+                )
 
         # Files referred to exactly are also ignored if
         # matched, but we warn the users when that happens
@@ -1003,7 +1020,7 @@ class Linter:
                     with open(fpath) as fh:
                         spec = pathspec.PathSpec.from_lines("gitwildmatch", fh)
                         ignores[dirpath] = spec
-                    # We don't need to process the ignore file any futher
+                    # We don't need to process the ignore file any further
                     continue
 
                 # We won't purge files *here* because there's an edge case
@@ -1199,7 +1216,7 @@ class Linter:
                 self.formatter.dispatch_path(path)
             # Load the file with the config and yield the result.
             try:
-                raw_file, config, encoding = self._load_raw_file_and_config(
+                raw_file, config, encoding = self.load_raw_file_and_config(
                     fname, self.config
                 )
             except SQLFluffSkipFile as s:
