@@ -85,7 +85,7 @@ class PathStep:
     Attributes:
         segment (:obj:`BaseSegment`): The segment in the chain.
         idx (int): The index of the target within its `segment`.
-        len (int): The number of childen `segment` has.
+        len (int): The number of children `segment` has.
     """
 
     segment: "BaseSegment"
@@ -864,6 +864,16 @@ class BaseSegment(metaclass=SegmentMetaclass):
         """Returns the type of this segment as a string."""
         return self.type
 
+    def count_segments(self, raw_only=False):
+        """Returns the number of segments in this segment."""
+        if self.segments:
+            self_count = 0 if raw_only else 1
+            return self_count + sum(
+                seg.count_segments(raw_only=raw_only) for seg in self.segments
+            )
+        else:
+            return 1
+
     def is_type(self, *seg_type):
         """Is this segment (or its parent) of the given type."""
         return self.class_is_type(*seg_type)
@@ -1074,7 +1084,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
         self,
         *seg_type: str,
         recurse_into: bool = True,
-        no_recursive_seg_type: str = None,
+        no_recursive_seg_type: Optional[str] = None,
     ):
         """Recursively crawl for segments of a given type.
 
@@ -1637,7 +1647,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
                     self.pos_marker.templated_slice.stop,
                 )
                 # We determine the source_slice directly rather than
-                # infering it so that we can be very specific that
+                # inferring it so that we can be very specific that
                 # we ensure that fixes adjacent to source-only slices
                 # (e.g. {% endif %}) are placed appropriately relative
                 # to source-only slices.
@@ -1670,8 +1680,8 @@ class BracketedSegment(BaseSegment):
         # be tuples of length 1. This is because we'll almost always
         # be doing tuple arithmetic with the results and constructing
         # 1-tuples on the fly is very easy to misread.
-        start_bracket: Tuple[BaseSegment] = None,
-        end_bracket: Tuple[BaseSegment] = None,
+        start_bracket: Optional[Tuple[BaseSegment]] = None,
+        end_bracket: Optional[Tuple[BaseSegment]] = None,
         **kwargs,
     ):
         """Stash the bracket segments for later."""
@@ -1685,7 +1695,7 @@ class BracketedSegment(BaseSegment):
 
     @classmethod
     def simple(cls, parse_context: ParseContext, crumbs=None) -> Optional[List[str]]:
-        """Simple methods for bracketed and the persitent brackets."""
+        """Simple methods for bracketed and the persistent brackets."""
         start_brackets = [
             start_bracket
             for _, start_bracket, _, persistent in parse_context.dialect.sets(
