@@ -61,7 +61,6 @@ OF
 ON
 OR
 ORDER
-PARTITION
 QUALIFY
 REGEXP
 REVOKE
@@ -102,6 +101,7 @@ ADD
 ADMIN
 AFTER
 ALLOW_DUPLICATE
+ALLOWED_IP_LIST
 ALLOW_OVERLAPPING_EXECUTION
 API
 API_INTEGRATION
@@ -145,6 +145,7 @@ BINARY_AS_TEXT
 BINARY_FORMAT
 BINDING
 BLOCK
+BLOCKED_IP_LIST
 BROTLI
 BZ2
 CACHE
@@ -381,6 +382,7 @@ OWNERSHIP
 PARALLEL
 PARAMETERS
 PARQUET
+PARTITION
 PASSWORD
 PAST
 PATTERN
