@@ -226,9 +226,11 @@ sparksql_dialect.replace(
         Sequence("DISTRIBUTE", "BY"),
         Sequence("SORT", "BY"),
         "HAVING",
+        "QUALIFY",
         Ref("SetOperatorSegment"),
         Ref("WithNoSchemaBindingClauseSegment"),
         Ref("WithDataClauseSegment"),
+        "KEYS",
     ),
     TemporaryGrammar=Sequence(
         Sequence("GLOBAL", optional=True),
@@ -306,6 +308,7 @@ sparksql_dialect.replace(
         "QUALIFY",
         "WINDOW",
         "OVERLAPS",
+        "APPLY",
     ),
     GroupByClauseTerminatorGrammar=OneOf(
         Sequence(
@@ -343,6 +346,26 @@ sparksql_dialect.replace(
         # Add arrow operators for lambdas (e.g. aggregate)
         Ref("RightArrowOperator"),
     ),
+    # Support for colon sign operator (Databricks SQL)
+    ObjectReferenceDelimiterGrammar=OneOf(
+        Ref("DotSegment"),
+        Sequence(Ref("DotSegment"), Ref("DotSegment")),
+        Ref("ColonSegment"),
+    ),
+    # Support for colon sign operator (Databricks SQL)
+    ObjectReferenceTerminatorGrammar=OneOf(
+        "ON",
+        "AS",
+        "USING",
+        Ref("CommaSegment"),
+        Ref("CastOperatorSegment"),
+        Ref("StartSquareBracketSegment"),
+        Ref("StartBracketSegment"),
+        Ref("BinaryOperatorGrammar"),
+        Ref("DelimiterGrammar"),
+        Ref("JoinLikeClauseGrammar"),
+        ansi.BracketedSegment,
+    ),
 )
 
 sparksql_dialect.add(
@@ -594,6 +617,18 @@ sparksql_dialect.add(
             type="signed_quoted_literal",
         ),
     ),
+    # Delta Live Tables CREATE TABLE and VIEW statements
+    OrRefreshGrammar=Sequence("OR", "REFRESH"),
+    # Databricks widget
+    WidgetNameIdentifierSegment=RegexParser(
+        r"[A-Z][A-Z0-9_]*",
+        CodeSegment,
+        type="widget_name_identifier",
+    ),
+    WidgetDefaultGrammar=Sequence(
+        "DEFAULT",
+        Ref("QuotedLiteralSegment"),
+    ),
 )
 
 # Adding Hint related grammar before comment `block_comment` and
@@ -929,7 +964,7 @@ class AlterTableStatementSegment(ansi.AlterTableStatementSegment):
                 "SET",
                 Ref("LocationGrammar"),
             ),
-            # ALTER TABLE - ADD/DROP CONTRAINTS (DELTA)
+            # ALTER TABLE - ADD/DROP CONSTRAINTS (DELTA)
             Sequence(
                 Indent,
                 OneOf("ADD", "DROP"),
@@ -1023,7 +1058,10 @@ class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
 
     match_grammar = Sequence(
         "CREATE",
-        Ref("OrReplaceGrammar", optional=True),
+        OneOf(Ref("OrReplaceGrammar"), Ref("OrRefreshGrammar"), optional=True),
+        Ref("TemporaryGrammar", optional=True),
+        Ref.keyword("STREAMING", optional=True),
+        Ref.keyword("LIVE", optional=True),
         "TABLE",
         Ref("IfNotExistsGrammar", optional=True),
         OneOf(
@@ -1092,8 +1130,10 @@ class CreateViewStatementSegment(ansi.CreateViewStatementSegment):
 
     match_grammar = Sequence(
         "CREATE",
-        Ref("OrReplaceGrammar", optional=True),
+        OneOf(Ref("OrReplaceGrammar"), Ref("OrRefreshGrammar"), optional=True),
         Ref("TemporaryGrammar", optional=True),
+        Ref.keyword("STREAMING", optional=True),
+        Ref.keyword("LIVE", optional=True),
         "VIEW",
         Ref("IfNotExistsGrammar", optional=True),
         Ref("TableReferenceSegment"),
@@ -1117,6 +1157,46 @@ class CreateViewStatementSegment(ansi.CreateViewStatementSegment):
     )
 
 
+class CreateWidgetStatementSegment(BaseSegment):
+    """A `CREATE WIDGET` STATEMENT.
+
+    https://docs.databricks.com/notebooks/widgets.html#databricks-widget-api
+    """
+
+    type = "create_widget_statement"
+
+    match_grammar = Sequence(
+        "CREATE",
+        "WIDGET",
+        OneOf(
+            Sequence(
+                "DROPDOWN",
+                Ref("WidgetNameIdentifierSegment"),
+                Ref("WidgetDefaultGrammar"),
+                Sequence("CHOICES", Ref("SelectStatementSegment")),
+            ),
+            Sequence(
+                "TEXT", Ref("WidgetNameIdentifierSegment"), Ref("WidgetDefaultGrammar")
+            ),
+        ),
+    )
+
+
+class RemoveWidgetStatementSegment(BaseSegment):
+    """A `REMOVE WIDGET` STATEMENT.
+
+    https://docs.databricks.com/notebooks/widgets.html#databricks-widget-api
+    """
+
+    type = "remove_widget_statement"
+
+    match_grammar = Sequence(
+        "REMOVE",
+        "WIDGET",
+        Ref("WidgetNameIdentifierSegment"),
+    )
+
+
 class DropFunctionStatementSegment(BaseSegment):
     """A `DROP FUNCTION` STATEMENT.
 
@@ -1855,7 +1935,7 @@ class TransformClauseSegment(BaseSegment):
 class ExplainStatementSegment(ansi.ExplainStatementSegment):
     """An `Explain` statement.
 
-    Enhanced from ANSI dialect to allow for additonal parameters.
+    Enhanced from ANSI dialect to allow for additional parameters.
 
     EXPLAIN [ EXTENDED | CODEGEN | COST | FORMATTED ] explainable_stmt
 
@@ -2372,6 +2452,12 @@ class StatementSegment(ansi.StatementSegment):
             Ref("GenerateManifestFileStatementSegment"),
             Ref("ConvertToDeltaStatementSegment"),
             Ref("RestoreTableStatementSegment"),
+            # Databricks - Delta Live Tables
+            Ref("ConstraintStatementSegment"),
+            Ref("ApplyChangesIntoStatementSegment"),
+            # Databricks - widgets
+            Ref("CreateWidgetStatementSegment"),
+            Ref("RemoveWidgetStatementSegment"),
         ],
         remove=[
             Ref("TransactionStatementSegment"),
@@ -2475,6 +2561,8 @@ class AliasExpressionSegment(ansi.AliasExpressionSegment):
                 Ref("JoinTypeKeywords"),
                 "WINDOW",
                 "PIVOT",
+                "KEYS",
+                "FROM",
             ),
         ),
     )
@@ -2701,8 +2789,8 @@ class UpdateStatementSegment(ansi.UpdateStatementSegment):
             Ref("FileReferenceSegment"),
             Ref("TableReferenceSegment"),
         ),
-        # SET is not a resevered word in all dialects (e.g. RedShift)
-        # So specifically exclude as an allowed implict alias to avoid parsing errors
+        # SET is not a reserved word in all dialects (e.g. RedShift)
+        # So specifically exclude as an allowed implicit alias to avoid parsing errors
         Ref(
             "AliasExpressionSegment",
             exclude=Ref.keyword("SET"),
@@ -2887,3 +2975,141 @@ class RestoreTableStatementSegment(BaseSegment):
             Ref("VersionAsOfGrammar"),
         ),
     )
+
+
+class ConstraintStatementSegment(BaseSegment):
+    """A `CONSTRAINT` statement to to define data quality on data contents.
+
+    https://docs.databricks.com/workflows/delta-live-tables/delta-live-tables-expectations.html#manage-data-quality-with-delta-live-tables
+    """
+
+    type = "constraint_statement"
+
+    match_grammar: Matchable = Sequence(
+        "CONSTRAINT",
+        Ref("ObjectReferenceSegment"),
+        "EXPECT",
+        Bracketed(Ref("ExpressionSegment")),
+        Sequence("ON", "VIOLATION", optional=True),
+        OneOf(
+            Sequence("FAIL", "UPDATE"),
+            Sequence("DROP", "ROW"),
+            optional=True,
+        ),
+    )
+
+
+class ApplyChangesIntoStatementSegment(BaseSegment):
+    """A statement ingest CDC data a target table.
+
+    https://docs.databricks.com/workflows/delta-live-tables/delta-live-tables-cdc.html#sql
+    """
+
+    type = "apply_changes_into_statement"
+
+    match_grammar = Sequence(
+        Sequence(
+            "APPLY",
+            "CHANGES",
+            "INTO",
+        ),
+        Indent,
+        Ref("TableExpressionSegment"),
+        Dedent,
+        Ref("FromClauseSegment"),
+        Sequence(
+            "KEYS",
+            Indent,
+            Ref("BracketedColumnReferenceListGrammar"),
+            Dedent,
+        ),
+        Sequence("IGNORE", "NULL", "UPDATES", optional=True),
+        Ref("WhereClauseSegment", optional=True),
+        AnyNumberOf(
+            Sequence(
+                "APPLY",
+                "AS",
+                OneOf("DELETE", "TRUNCATE"),
+                "WHEN",
+                Ref("ColumnReferenceSegment"),
+                Ref("EqualsSegment"),
+                Ref("QuotedLiteralSegment"),
+            ),
+            # NB: Setting max_times to allow for one instance
+            #     of DELETE and TRUNCATE at most
+            max_times=2,
+        ),
+        Sequence(
+            "SEQUENCE",
+            "BY",
+            Ref("ColumnReferenceSegment"),
+        ),
+        Sequence(
+            "COLUMNS",
+            OneOf(
+                Delimited(
+                    Ref("ColumnReferenceSegment"),
+                ),
+                Sequence(
+                    Ref("StarSegment"),
+                    "EXCEPT",
+                    Ref("BracketedColumnReferenceListGrammar"),
+                ),
+            ),
+        ),
+        Sequence(
+            "STORED",
+            "AS",
+            "SCD",
+            "TYPE",
+            Ref("NumericLiteralSegment"),
+            optional=True,
+        ),
+    )
+
+
+class WildcardExpressionSegment(ansi.WildcardExpressionSegment):
+    """An extension of the star expression for Databricks."""
+
+    match_grammar = ansi.WildcardExpressionSegment.match_grammar.copy(
+        insert=[
+            # Optional EXCEPT clause
+            # https://docs.databricks.com/release-notes/runtime/9.0.html#exclude-columns-in-select--public-preview
+            Ref("ExceptClauseSegment", optional=True),
+        ]
+    )
+
+
+class ExceptClauseSegment(BaseSegment):
+    """SELECT * EXCEPT clause."""
+
+    type = "select_except_clause"
+    match_grammar = Sequence(
+        "EXCEPT",
+        Bracketed(Delimited(Ref("SingleIdentifierGrammar"))),
+    )
+
+
+class SelectClauseSegment(BaseSegment):
+    """A group of elements in a select target statement.
+
+    It's very similar to `SelectClauseSegment` from `dialect_ansi` except does not
+    have set `SetOperatorSegment` as possible terminator - this is to avoid issues
+    with wrongly recognized `EXCEPT`.
+    """
+
+    type = "select_clause"
+    match_grammar: Matchable = StartsWith(
+        "SELECT",
+        terminator=OneOf(
+            "FROM",
+            "WHERE",
+            "UNION",
+            Sequence("ORDER", "BY"),
+            "LIMIT",
+            "OVERLAPS",
+        ),
+        enforce_whitespace_preceding_terminator=True,
+    )
+
+    parse_grammar: Matchable = Ref("SelectClauseSegmentGrammar")
